SystemOrganization addCategory: #'MorphoPhysics-Application'!
SystemOrganization addCategory: #MorphoPhysics!

PolygonMorph subclass: #ForceMorph
	instanceVariableNames: 'ball1 ball2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphoPhysics'!

ForceMorph subclass: #ElectricForceMorph
	instanceVariableNames: 'force'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphoPhysics'!

!ElectricForceMorph methodsFor: 'physics' stamp: 'hjo 4/26/2009 18:09'!
applyForceBothEnds: aPoint

	super applyForceBothEnds: aPoint.
	self force: aPoint! !

!ElectricForceMorph methodsFor: 'defaults' stamp: 'hjo 4/26/2009 18:08'!
defaultForce

	^ 0@0! !

!ElectricForceMorph methodsFor: 'accessing' stamp: 'hjo 4/26/2009 18:08'!
force

	^force setDefault: #defaultForce for: self withSetter: #force:! !

!ElectricForceMorph methodsFor: 'accessing' stamp: 'hjo 4/26/2009 18:08'!
force: anObject
		
	force := anObject! !

!ElectricForceMorph methodsFor: 'physics' stamp: 'hjo 4/21/2009 21:29'!
forceFromDisplacementVector: aPoint

	^aPoint / (aPoint r raisedTo: 3) * 1000000! !

!ElectricForceMorph methodsFor: 'initialization' stamp: 'hjo 4/21/2009 14:35'!
initialize

	super initialize.
	self makeOpen.
	self borderColor: Color blue lighter.! !

!ElectricForceMorph methodsFor: 'physics' stamp: 'hjo 4/26/2009 18:19'!
updateAppearance

	self borderWidth: self force r! !

!ForceMorph classSide methodsFor: 'instance creation' stamp: 'hjo 4/21/2009 21:13'!
newConnecting: aBallMorph and: aBallMorph2 

	^self new
		ball1: aBallMorph;
		ball2: aBallMorph2! !

!ForceMorph methodsFor: 'physics' stamp: 'hjo 4/21/2009 21:33'!
applyForce

	self applyForceBothEnds: (self forceFromDisplacementVector: self ballDisplacementVector)! !

!ForceMorph methodsFor: 'physics' stamp: 'hjo 4/21/2009 21:00'!
applyForceBothEnds: aPoint 

	ball1 forcedBy: aPoint.
	ball2 forcedBy: aPoint negated! !

!ForceMorph methodsFor: 'physics' stamp: 'hjo 4/21/2009 21:00'!
attachTo: aBallMorph and: aBallMorph2 

	self setVertices: (Array with: aBallMorph center with: aBallMorph2 center).
! !

!ForceMorph methodsFor: 'accessing' stamp: 'hjo 4/27/2009 23:58'!
ball1

	^ball1 setDefault: #defaultBall1 for: self withSetter: #ball1:! !

!ForceMorph methodsFor: 'accessing' stamp: 'hjo 4/21/2009 21:00'!
ball1: aBallMorph 

	ball1 _ aBallMorph! !

!ForceMorph methodsFor: 'accessing' stamp: 'hjo 4/27/2009 23:58'!
ball2

	^ball2 setDefault: #defaultBall2 for: self withSetter: #ball2:! !

!ForceMorph methodsFor: 'accessing' stamp: 'hjo 4/21/2009 21:00'!
ball2: aBallMorph 

	ball2 _ aBallMorph! !

!ForceMorph methodsFor: 'physics' stamp: 'hjo 4/21/2009 21:00'!
ballDisplacementVector

	^ball1 center - ball2 center! !

!ForceMorph methodsFor: 'physics' stamp: 'hjo 4/21/2009 21:19'!
currentLength

	^self ballDisplacementVector r! !

!ForceMorph methodsFor: 'defaults' stamp: 'hjo 4/27/2009 23:59'!
defaultBall1

	^UnmovableBodyMorph new openInWorld
		center: self vertices first! !

!ForceMorph methodsFor: 'defaults' stamp: 'hjo 4/28/2009 00:00'!
defaultBall2

	^UnmovableBodyMorph new openInWorld
		center: self vertices first! !

!ForceMorph methodsFor: 'testing' stamp: 'hjo 4/26/2009 18:07'!
initialize

	super initialize.
	self makeNoArrows! !

!ForceMorph methodsFor: 'stepping and presenter' stamp: 'hjo 4/27/2009 23:59'!
step

	super step.
	self attachTo: self ball1 and: self ball2.
	self applyForce.
	self updateAppearance! !

!ForceMorph methodsFor: 'testing' stamp: 'hjo 4/21/2009 21:00'!
stepTime

	^17! !

!ForceMorph methodsFor: 'physics' stamp: 'hjo 4/26/2009 18:02'!
updateAppearance

	"do something to change the appearance of me.
	#step calls me."! !

ForceMorph subclass: #RubberBandMorph
	instanceVariableNames: 'stableLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphoPhysics'!

!RubberBandMorph classSide methodsFor: 'eToys-scripting' stamp: 'hjo 4/28/2009 10:57'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^
	#(
		(rubberBand 
			(
				(slot stableLength 'Length that I get easy with.' Number readWrite Player getStableLength Player setStableLength: )
			)
		)
	)


! !

!RubberBandMorph methodsFor: 'defaults' stamp: 'hjo 4/21/2009 11:21'!
defaultStableLength

	"Return a default object for instance variable stableLength."
	^200! !

!RubberBandMorph methodsFor: 'physics' stamp: 'hjo 4/27/2009 23:32'!
forceFromDisplacementVector: aPoint

	| dist |
	dist _ aPoint r.
	^dist isZero
		ifTrue: [ Point r: 1 degrees: 360 atRandom ]
		ifFalse: [ aPoint / dist * (self stableLength - dist) ]! !

!RubberBandMorph methodsFor: 'initialization' stamp: 'hjo 4/21/2009 11:15'!
initialize

	super initialize.
	self makeOpen.
	self borderColor: Color black.! !

!RubberBandMorph methodsFor: 'accessing' stamp: 'hjo 4/21/2009 11:21'!
stableLength

	stableLength setDefault: #defaultStableLength for: self withSetter: #stableLength: .
	^stableLength! !

!RubberBandMorph methodsFor: 'accessing' stamp: 'hjo 4/21/2009 11:21'!
stableLength: anObject
		
	stableLength := anObject! !

!RubberBandMorph methodsFor: 'physics' stamp: 'hjo 4/27/2009 23:38'!
updateAppearance

	| length |
	length := self currentLength.
	length isZero
		ifTrue: [ "do nothing" ]
		ifFalse: [ self borderWidth: ((self stableLength / length)*3 max: 1) ]! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 4/4/2009 18:02'!
accelerateXBy: aNumber

	costume acceleratedBy: aNumber@0! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 4/4/2009 18:03'!
accelerateYBy: aNumber

	costume acceleratedBy: 0@aNumber! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 3/12/2009 17:14'!
getMass

	^costume mass! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 4/28/2009 10:58'!
getStableLength

	^costume stableLength! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 4/4/2009 09:59'!
getVelocityX

	^costume velocity x! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 4/4/2009 10:03'!
getVelocityY

	^costume velocity y! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 3/12/2009 17:14'!
setMass: aNumber

	costume mass: aNumber! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 4/28/2009 10:58'!
setStableLength: aNumber

	costume stableLength: aNumber! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 4/4/2009 10:00'!
setVelocityX: aNumber

	^costume velocity: (aNumber @ costume velocity y)! !

!Player methodsFor: '*MorphoPhysics' stamp: 'hjo 4/4/2009 10:03'!
setVelocityY: aNumber

	^costume velocity: (costume velocity x @ aNumber)! !

EllipseMorph subclass: #BallMorph
	instanceVariableNames: 'velocity mass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphoPhysics'!

!BallMorph classSide methodsFor: 'eToys-scripting' stamp: 'hjo 4/28/2009 10:48'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^
	#(
		(rigitBody 
			(
				(slot mass 'How heavy am I?' Number readWrite Player getMass Player setMass: )
				(slot velocityX 'moving per time' Number readWrite Player getVelocityX Player setVelocityX: )
				(slot velocityY 'moving per time' Number readWrite Player getVelocityY Player setVelocityY: )
				(command accelerateXBy: 'Give me a little push to x' Number)
				(command accelerateYBy: 'Give me a little push to y' Number)
			)
		)
	)


! !

!BallMorph methodsFor: 'physics' stamp: 'hjo 4/28/2009 00:09'!
acceleratedBy: aPoint 

	self velocity: self velocity + aPoint! !

!BallMorph methodsFor: 'physics' stamp: 'hjo 4/21/2009 10:56'!
applyFriction

	self velocity: self velocity * 0.98! !

!BallMorph methodsFor: 'defaults' stamp: 'hjo 4/21/2009 11:07'!
defaultMass

	"Return a default object for instance variable mass."
	^500! !

!BallMorph methodsFor: 'defaults' stamp: 'hjo 4/21/2009 10:54'!
defaultVelocity

	"Return a default object for instance variable velocity."
	^0@0! !

!BallMorph methodsFor: 'physics' stamp: 'hjo 4/28/2009 00:09'!
forcedBy: aPoint 

	self acceleratedBy: aPoint / self mass! !

!BallMorph methodsFor: 'initialization' stamp: 'hjo 4/28/2009 11:46'!
initialize

	super initialize.
	self borderWidth: 0.
	self extent: 40@40! !

!BallMorph methodsFor: 'accessing' stamp: 'hjo 4/28/2009 00:08'!
mass

	^mass setDefault: #defaultMass for: self withSetter: #mass:! !

!BallMorph methodsFor: 'accessing' stamp: 'hjo 4/21/2009 11:06'!
mass: anObject
		
	mass := anObject! !

!BallMorph methodsFor: 'stepping and presenter' stamp: 'hjo 4/21/2009 10:56'!
step

	super step.
	self bounds: (self bounds translateBy: self velocity).
	self applyFriction! !

!BallMorph methodsFor: 'testing' stamp: 'hjo 4/21/2009 10:53'!
stepTime

	^17! !

!BallMorph methodsFor: 'accessing' stamp: 'hjo 4/28/2009 00:08'!
velocity

	^velocity setDefault: #defaultVelocity for: self withSetter: #velocity:! !

!BallMorph methodsFor: 'accessing' stamp: 'hjo 4/21/2009 10:54'!
velocity: anObject
		
	velocity := anObject! !

EllipseMorph subclass: #UnmovableBodyMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphoPhysics'!

!UnmovableBodyMorph methodsFor: 'physics' stamp: 'hjo 3/7/2009 13:12'!
acceleratedBy: aPoint 

	"I shall not move!!!!"! !

!UnmovableBodyMorph methodsFor: 'physics' stamp: 'hjo 3/7/2009 13:13'!
forcedBy: aPoint 

	"I shall not move!!!!"! !

!UnmovableBodyMorph methodsFor: 'as yet unclassified' stamp: 'hjo 3/7/2009 15:34'!
initialize

	super initialize.
	self extent: 20@20! !

Object subclass: #IdeaMap
	instanceVariableNames: 'balls electricForceMorphs rubberBandMorphs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphoPhysics-Application'!

!IdeaMap classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/7/2009 18:27'!
example

"e _IdeaMap new .
e deleteMorphs.
e addNewBall.
e addNewBallRubberBandConnectedToAll: e balls.
e addNewBallRubberBandConnectedToAll: {e balls at: 3}
e electricForceMorphs do: [:each| each hide ]
ElectricForceMorph allInstances do: [:each| each hide]"! !

!IdeaMap methodsFor: 'as yet unclassified' stamp: 'hjo 5/4/2009 13:21'!
addNewBall

	| b |
	b := BallMorph new.
	b position: 100 atRandom @ 100 atRandom + 100.
	b openInWorld.
	self balls do: [:each| | e | e := ElectricForceMorph newConnecting: each and: b. e openInWorld.  self electricForceMorphs add: e. e hide ].
	self balls add: b.
	^b! !

!IdeaMap methodsFor: 'as yet unclassified' stamp: 'hjo 5/4/2009 01:17'!
addNewBallRubberBandConnectedToAll: anArray

	| b |
	(self balls includesAllOf: anArray) ifFalse: [ self error: 'Morphs to connect rubber must be included in balls.' ].
	b := self addNewBall.
	anArray do: [:each| |r| r := RubberBandMorph newConnecting: each and: b. r openInWorld. self rubberBandMorphs add: r. r goBehind ].
	^b
	! !

!IdeaMap methodsFor: 'accessing' stamp: 'hjo 5/4/2009 00:47'!
balls

	^balls setDefault: #defaultBalls for: self withSetter: #balls:! !

!IdeaMap methodsFor: 'accessing' stamp: 'hjo 5/4/2009 00:47'!
balls: anObject
		
	balls := anObject! !

!IdeaMap methodsFor: 'defaults' stamp: 'hjo 5/4/2009 00:47'!
defaultBalls

	^OrderedCollection new! !

!IdeaMap methodsFor: 'defaults' stamp: 'hjo 5/4/2009 00:51'!
defaultElectricForceMorphs

	^OrderedCollection new! !

!IdeaMap methodsFor: 'defaults' stamp: 'hjo 5/4/2009 00:51'!
defaultRubberBandMorphs

	^OrderedCollection new! !

!IdeaMap methodsFor: 'as yet unclassified' stamp: 'hjo 5/4/2009 00:53'!
deleteMorphs

	self balls do: [:each| each delete ].
	self balls: nil.
	self electricForceMorphs do: [:each| each delete ].
	self electricForceMorphs: nil.
	self rubberBandMorphs do: [:each| each delete ].
	self rubberBandMorphs: nil! !

!IdeaMap methodsFor: 'accessing' stamp: 'hjo 5/4/2009 00:51'!
electricForceMorphs

	^electricForceMorphs setDefault: #defaultElectricForceMorphs for: self withSetter: #electricForceMorphs:! !

!IdeaMap methodsFor: 'accessing' stamp: 'hjo 5/4/2009 00:51'!
electricForceMorphs: anObject
		
	electricForceMorphs := anObject! !

!IdeaMap methodsFor: 'accessing' stamp: 'hjo 5/4/2009 00:51'!
rubberBandMorphs

	^rubberBandMorphs setDefault: #defaultRubberBandMorphs for: self withSetter: #rubberBandMorphs:! !

!IdeaMap methodsFor: 'accessing' stamp: 'hjo 5/4/2009 00:51'!
rubberBandMorphs: anObject
		
	rubberBandMorphs := anObject! !

Object subclass: #RubberMesh
	instanceVariableNames: 'b1 b2 b3 b4 c s1 s2 s3 s4 i1 i2 i3 i4'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphoPhysics-Application'!

!RubberMesh methodsFor: 'as yet unclassified' stamp: 'hjo 4/28/2009 00:29'!
deleteMorphs

	{b1. b2. b3. b4. c. s1. s2. s3. s4. i1. i2. i3. i4} do: [:morph| morph delete ].! !

!RubberMesh methodsFor: 'as yet unclassified' stamp: 'hjo 4/28/2009 00:41'!
openMorphs

	| sideLength centerLength |
	sideLength := 2 * 100.
	centerLength := 2 sqrt * 100.
	b1 := BallMorph new openInWorld.
	b2 := BallMorph new openInWorld.
	b3 := BallMorph new openInWorld.
	b4 := BallMorph new openInWorld.
	c := BallMorph new openInWorld.

	s1 := RubberBandMorph newConnecting: b1 and: b2.
	s1 stableLength: sideLength.
	s1 openInWorld.

	s2 := RubberBandMorph newConnecting: b2 and: b3.
	s2 stableLength: sideLength.
	s2 openInWorld.

	s3 := RubberBandMorph newConnecting: b3 and: b4.
	s3 stableLength: sideLength.
	s3 openInWorld.

	s4 := RubberBandMorph newConnecting: b4 and: b1.
	s4 stableLength: sideLength.
	s4 openInWorld.

	i1 := RubberBandMorph newConnecting: b1 and: c.
	i1 stableLength: centerLength.
	i1 openInWorld.

	i2 := RubberBandMorph newConnecting: b2 and: c.
	i2 stableLength: centerLength.
	i2 openInWorld.

	i3 := RubberBandMorph newConnecting: b3 and: c.
	i3 stableLength: centerLength.
	i3 openInWorld.

	i4 := RubberBandMorph newConnecting: b4 and: c.
	i4 stableLength: centerLength.
	i4 openInWorld.

	b1 position: 200@200.
	b2 position: 300@200.
	b3 position: 300@300.
	b4 position: 200@300.
	{b1. b2. b3. b4. c} do: [:each| each comeToFront ].
	c position: (b1 position + b2 position + b3 position + b4 position) / 4.

! !
